<?php
/**
 * Moi (Luc Sorel, http://www.lucsorel.com/index.php?page=contact) reconnaît être l'auteur de ce code source. 
 * J'ai placé ce code source dans le domaine publique. Vous pouvez utiliser, modifier et distribuer le code 
 * source et les programmes exécutables basés sur ce code source. Veuillez toutefois prendre en considération 
 * les points suivants :
 * 
 * --- Décharge légale de garantie ---
 * Ce code source vous est présenté "en tant que tel" et sans garantie de performance ou d'intérêt commercial. 
 * L'auteur et / ou le distributeur ont pu faire des déclarations ) propos de ce code source. Ces déclarations 
 * n'ont pas valeur de garanties et ne doivent pas servir de fondements quant à l'utilisation tierce de ce code 
 * source.
 * 
 * Ce code source est mis à disposition sans aucune garantie implicite ou explicite. Étant donné la diversité 
 * des conditions contextuelles et matérielles sous lesquelles ce code source peut être utilisé, aucune garantie 
 * d'adéquation pour un usage spécifique n'est proposé. il est conseillé que l'utilisateur teste le code source 
 * consciencieusement avant de s'appuyer dessus. L'utilisateur prend l'entière responsabilité de l'utilisation 
 * de ce code source.
 */

require_once( 'Utilisateur.inc' );
require_once( 'JSONTools.inc' );

class Controleur
{
 	/**
 	 * @var	Controleur	variable statique utilisée par le modèle de conception singleton
 	 */
 	static private $sC_controleur;

 	private function __construct()
	{}

	static public function Instance()
	{
		if( ! isset($sC_controleur) )
		{
			$sC_controleur = new Controleur();
		}
		return $sC_controleur;
	}

	public function traiter_requete()
	{
		// détermination de l'action à mener
		$ls_action = $_REQUEST[ 'action' ];
		$ls_methode_action = 'action_' . $ls_action;

		// envoi d'un message d'erreur si l'action demandée n'est pas gérée
		if( ! method_exists($this, $ls_methode_action) )
		{
			throw new Exception( "L'action '$ls_action' n'est pas gérée" );
		}
		// validations diverses avant l'exécution de l'action demandée
		else 
		{
			// vérification de la session pour les action différentes que "connecter" et "déconnecter" et message d'erreur sinon
			if( ($ls_methode_action != 'action_connecter') && ($ls_methode_action != 'action_deconnecter')
			&& ! $this->session_hesperis_en_cours() )
			{
				throw new Exception( "Vous n'avez pas le droit de réaliser l'action '$ls_action'" );
			}

			// exécution de l'action demandée
			$this->$ls_methode_action();
		}
	}

	protected function action_connecter()
	{
		// récupération du compte et mot de passe
		$ls_login = $_REQUEST['login'];
		$ls_password = $_REQUEST['password'];

		// message d'erreur si l'un d'entre eux n'est pas renseigné
		if( empty($ls_login) || empty($ls_password) )
		{
			throw new Exception( "Compte ou mot de passe non renseigné" );
		}
		// vérification du compte et mot de passe
		else
		{
			// salage et cryptage du mot de passe
			$ls_password_crypte = md5( 'hesperis_' . $ls_password );

			/* si le login-mot de passe ont pu être validés (à implémenter) :
			 * - on commence une session Hespéris
			 * - on renvoie la linéarisation JSON de l'utilisateur en retour de requête
			 */
			if( $lU_utilisateur = Utilisateur::Instancier_selon_login_password_crypte(
				$ls_login,$ls_password_crypte) )
			{
				// initialiser session
				$this->commencer_session_selon_login( $ls_login );
				// renvoi de l'utilisateur au format JSON
				die( $lU_utilisateur->exporter_en_JSON() );
			}
			// sinon on efface l'éventuelle session en cours et on affiche un message d'explication en retour de requÃªte
			else
			{
				$this->stop_session_hesperis();
				throw new Exception( "Connexion refusée : login ('$ls_login') ou mot de passe erroné" );
			}
		}
	}

	/**
	 * Termine l'éventuelle session en cours et renvoie 'true'
	 */
	private function action_deconnecter()
	{
		$this->stop_session_hesperis();
		die( 'true' );
	}

	public function lancer_erreur_json( $ps_message_erreur )
	{
		// affichage du message d'erreur et fin de l'activité du serveur
		die( '{"Erreur":{"message":' . JSONTools::Proteger_chaine_pour_JSON($ps_message_erreur) . '}}' );
	}

	private function action_truc()
	{
		throw new Exception( "j'ai fait un truc" );
	}

	private function commencer_session_selon_login( $ps_login )
	{
		// vérification que le login est une chaÃ®ne de caractères non vide
		if( ! is_string($ps_login) || empty($ps_login) )
		{
			$this->stop_session_hesperis();
			throw new Exception( "Impossible de commencer une session avec un login invalide : '$ps_login'" );
		}

		// régénération de l'identifiant de session pour éviter le vol de session
		session_regenerate_id( true );

		// mise en session du login de l'utilisateur connecté
		$_SESSION[ 'hesperis_login' ] = $ps_login;
	}

	private function session_hesperis_en_cours()
	{
		return isset( $_SESSION['hesperis_login'] );
	}

	private function stop_session_hesperis()
	{
		// 1. destruction légère au cas où on ait envie de se servir encore d'autres variables de session après la déconnexion de l'utilisateur
		//unset( $_SESSION['hesperis_login'] );
		// 2. destruction lourde : on supprime le cookie de session et la variable de session
		session_destroy();
		unset( $_SESSION );
	}
}