/**
 * Moi (Luc Sorel, http://www.lucsorel.com/index.php?page=contact) reconnaît être l'auteur de ce code source. 
 * J'ai placé ce code source dans le domaine publique. Vous pouvez utiliser, modifier et distribuer le code 
 * source et les programmes exécutables basés sur ce code source. Veuillez toutefois prendre en considération 
 * les points suivants :
 * 
 * --- Décharge légale de garantie ---
 * Ce code source vous est présenté "en tant que tel" et sans garantie de performance ou d'intérêt commercial. 
 * L'auteur et / ou le distributeur ont pu faire des déclarations ) propos de ce code source. Ces déclarations 
 * n'ont pas valeur de garanties et ne doivent pas servir de fondements quant à l'utilisation tierce de ce code 
 * source.
 * 
 * Ce code source est mis à disposition sans aucune garantie implicite ou explicite. Étant donné la diversité 
 * des conditions contextuelles et matérielles sous lesquelles ce code source peut être utilisé, aucune garantie 
 * d'adéquation pour un usage spécifique n'est proposé. il est conseillé que l'utilisateur teste le code source 
 * consciencieusement avant de s'appuyer dessus. L'utilisateur prend l'entière responsabilité de l'utilisation 
 * de ce code source.
 */

package com.lucsorel.hesperis.client.presview;

import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.dom.client.HasKeyUpHandlers;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HasText;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.PasswordTextBox;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.ui.HorizontalPanel;

import com.lucsorel.hesperis.client.presview.IConnexion.Display;

public class ConnexionDisplay extends Composite implements Display
{
	/**
	 * Déclaration des éléments graphiques composant la vue, et qui pourront être renvoyés 
	 * par le widget sous forme de ...HasHandlers. Ces méthodes typées ...HasHandler doivent
	 * être déclarées dans l'interface IConnexion associée.
	 */
	// Panneau englobant le widget (permet une initialisation du widget dans le constructeur)
	private HorizontalPanel iHP_connexionPanel;
	
	final SimplePanel iSP_message;
	final TextBox iTB_login;
	final PasswordTextBox iPTB_motDePasse;
	final Button iB_connexion;

	/**
	 * Le module GIN a besoin d'un constructeur sans paramètre 
	 * pour pouvoir faire de l'injection de dépendances.
	 */
	public ConnexionDisplay()
	{
		iHP_connexionPanel = new HorizontalPanel();
		
		// déclarer et initialiser les autres widgets éventuels
		/*
		 *  Création du widget
		 */
		// 1. création du panneau général de la vue
		iHP_connexionPanel = new HorizontalPanel();
		iHP_connexionPanel.setStylePrimaryName( "connexion-global" );

		// 2. création du panneau contenant les champs de connexion
		VerticalPanel lVP_loginPanel = new VerticalPanel();
		lVP_loginPanel.setStylePrimaryName( "login" );

		// 2.1 compte
		HorizontalPanel lHP_loginCompte = new HorizontalPanel();
		lHP_loginCompte.add( new Label("Compte : ") );
		iTB_login = new TextBox();
		lHP_loginCompte.add( iTB_login );

		// 2.2 mot de passe
		HorizontalPanel lHP_loginMotDePasse = new HorizontalPanel();
		lHP_loginMotDePasse.add( new Label("Mot de passe : ") );
		iPTB_motDePasse = new PasswordTextBox();
		lHP_loginMotDePasse.add( iPTB_motDePasse );

		// 2.3 bouton de connexion
		iB_connexion = new Button( "Me connecter" );

		// 3. accueil du panneau d'affichage de messages
		iSP_message = new SimplePanel();

		// 4. assemblage de l'interface
		lVP_loginPanel.add( lHP_loginCompte );
		lVP_loginPanel.add( lHP_loginMotDePasse );
		lVP_loginPanel.add( iB_connexion );
		iHP_connexionPanel.add( lVP_loginPanel );
		iHP_connexionPanel.add( iSP_message );

		// définit "iHP_connexionPanel" comme étant le composant graphique représentant cette classe
		initWidget( iHP_connexionPanel );
	}
	
	/**
	 * Méthode surchargée pour l'implémentation du pattern MVP
	 * @see org.enunes.gwt.mvp.client.view.Display#asWidget()
	 */
	@Override
	public Widget asWidget()
	{
		return this;
	}

	/**
	 * Renvoie le champ de saisie de login à travers l'interface de gestion
	 * de touche pressée
	 * @return	HasKeyUpHandlers
	 * 
	 * @see com.lucsorel.hesperis.client.presview.IConnexion.Display#getLoginKeyUpHandlers()
	 */
	public HasKeyUpHandlers getLoginKeyUpHandlers()
	{
		return this.iTB_login;
	}

	/**
	 * Renvoie le champ de saisie de mot de passe à travers l'interface de gestion
	 * de touche pressée
	 * @return	HasKeyUpHandlers
	 * 
	 * @see com.lucsorel.hesperis.client.presview.IConnexion.Display#getMotDePasseKeyUpHandlers()
	 */
	public HasKeyUpHandlers getMotDePasseKeyUpHandlers()
	{
		return this.iPTB_motDePasse;
	}

	/**
	 * Renvoie le bouton de connexion à travers l'interface de gestion de clic
	 * @return	HasClickHandlers
	 * 
	 * @see com.lucsorel.hesperis.client.presview.IConnexion.Display#getConnexionClickHandlers()
	 */
	public HasClickHandlers getConnexionClickHandlers()
	{
		return this.iB_connexion;
	}

	/**
	 * Renvoie le champ de saisie de login à travers l'interface de gestion de texte
	 * @return	HasText
	 * @see com.lucsorel.hesperis.client.presview.IConnexion.Display#getLoginTexte()
	 */
	@Override
	public HasText getLoginTexte()
	{
		return this.iTB_login;
	}

	/**
	 * Renvoie le champ de saisie de mot de passe à travers l'interface de gestion de texte
	 * @return	HasText
	 * @see com.lucsorel.hesperis.client.presview.IConnexion.Display#getMotDePasseTexte()
	 */
	@Override
	public HasText getMotDePasseTexte()
	{
		return this.iPTB_motDePasse;
	}

	/**
	 * Ajoute la vue du module d'affichage des messages dans le widget général de connexion
	 * @see com.lucsorel.hesperis.client.presview.IConnexion.Display#ajouterMessageDisplay(IMessage.Display)
	 */
	@Override
	public void ajouterMessageDisplay( IMessage.Display display )
	{
		this.iSP_message.setWidget( display.asWidget() );
	}
}